#include <msp/fs/dir.h>
#include <msp/fs/stat.h>
#include "collection.h"
#include "directorysource.h"

using namespace std;

namespace Msp {
namespace DataFile {

void DirectorySource::add_directory(const FS::Path &d, bool replace)
{
	for(const string &f: FS::list_files(d))
		if(!objects.count(f) || replace)
			objects[f] = d/f;
}

bool DirectorySource::is_loadable(const CollectionItemTypeBase &, const string &name) const
{
	return objects.count(name);
}

CollectionSource::NameList DirectorySource::get_names(const CollectionItemTypeBase &type) const
{
	NameList names;
	for(const auto &kvp: objects)
		if(type.match_name(kvp.first))
			names.push_back(kvp.first);
	return names;
}

void DirectorySource::load(Collection &coll, const CollectionItemTypeBase &type, const string &name) const
{
	FS::Path file;
	if(lookup_file(name, file))
	{
		IO::BufferedFile in(file.str());
		Parser parser(in, file.str());
		type.load_item(coll, parser, name);
	}
}

IO::Seekable *DirectorySource::open(const string &name) const
{
	FS::Path file;
	if(lookup_file(name, file))
		return new IO::BufferedFile(file.str());

	return 0;
}

bool DirectorySource::lookup_file(const string &name, FS::Path &result) const
{
	ObjectMap::const_iterator i = objects.find(name);
	if(i==objects.end())
		return false;

	result = i->second;
	return true;
}

} // namespace DataFile
} // namespace Msp
